--[[--
@module MyDungeonsBook
]]

--[[--
UI
@section UI
]]

local L = LibStub("AceLocale-3.0"):GetLocale("MyDungeonsBook");

local function getHealBySpellMenu(rows, index, cols, challengeId)
    local spellId = rows[index].cols[1].value;
    local report = MyDungeonsBook:HealByEnemiesFrame_Report_Create(rows[index], cols, challengeId);
    return {
        MyDungeonsBook:WowHead_Menu_SpellComplex(spellId),
        MyDungeonsBook:Report_Menu(report)
    };
end

function MyDungeonsBook:HealByEnemiesFrame_Create(parentFrame, challengeId)
    local healByEnemiesBySpellFrame = self:TabContentWrapperWidget_Create(parentFrame);
    local data = self:HealByEnemiesFrame_GetDataForTable(challengeId, "ALL-HEAL-DONE-BY-ENEMIES");
    local columns = self:HealByEnemiesFrame_GetHeadersForTable(challengeId);
    local table = self:TableWidget_Create(columns, 11, 40, nil, healByEnemiesBySpellFrame, "all-heal-done-by-enemies");
    table:SetData(data);
    table:RegisterEvents({
        OnClick = function(_, _, data, _, _, realrow, _, _, button)
            if (button == "RightButton" and realrow) then
                local spellId = data[realrow].cols[1].value;
                if (spellId > 0) then
                    EasyMenu(getHealBySpellMenu(data, realrow, table.cols, challengeId, unitId), self.menuFrame, "cursor", 0 , 0, "MENU");
                end
            end
        end,
        OnEnter = function (_, cellFrame, data, _, _, realrow, column)
            if (realrow) then
                if (column == 2 or column == 3) then
                    self:Table_Cell_SpellMouseHover(cellFrame, data[realrow].cols[1].value);
                end
            end
        end,
        OnLeave = function (_, _, _, _, _, realrow, column)
            if (realrow) then
                if (column == 2 or column == 3) then
                    self:Table_Cell_MouseOut();
                end
            end
        end
    });
    return healByEnemiesBySpellFrame;
end


local function proceedSpellStats(spellId, spellStats, summaryRow, unitName)
    local row = {
        cols = {
            {value = spellId},
            {value = spellId},
            {value = spellId},
            {value = spellStats.hits},
            {value = spellStats.amount},
            {value = 0},
            {value = spellStats.overheal},
            {value = (spellStats.hitsCrit or 0) / spellStats.hits},
            {value = spellStats.hitsCrit or 0},
            {value = (spellStats.maxCrit == 0 and "-") or spellStats.maxCrit},
            {value = (spellStats.minCrit == math.huge and "-") or spellStats.minCrit},
            {value = (spellStats.maxNotCrit == 0 and "-") or spellStats.maxNotCrit},
            {value = (spellStats.minNotCrit == math.huge and "-") or spellStats.minNotCrit},
        },
        meta = {
            unitName = unitName
        }
    };
    summaryRow.cols[4].value = summaryRow.cols[4].value + spellStats.hits;
    summaryRow.cols[5].value = summaryRow.cols[5].value + spellStats.amount;
    summaryRow.cols[7].value = summaryRow.cols[7].value + spellStats.overheal;
    summaryRow.cols[9].value = summaryRow.cols[9].value + spellStats.hitsCrit;
    return row, summaryRow;
end

function MyDungeonsBook:HealByEnemiesFrame_GetDataForTable(challengeId, key)
    local tableData = {};
    if (not challengeId) then
        return {};
    end
    local mechanics = self:Challenge_Mechanic_GetById(challengeId, key);
    if (not mechanics) then
        self:DebugPrint(string.format("No Heal Done By Enemies data for challenge #%s", challengeId));
        return tableData;
    end
    local summaryRow = {
        color = {
            r = 0,
            g = 100,
            b = 0,
            a = 1
        },
        cols = {
            {value = -1},
            {value = -1},
            {value = -1},
            {value = 0},
            {value = 0},
            {value = ""},
            {value = 0},
            {value = 0},
            {value = 0},
            {value = ""},
            {value = ""},
            {value = ""},
            {value = ""}
        }
    };
    for spellId, spellStats in pairs(mechanics.spells) do
        if (spellId ~= "meta") then
            local row = proceedSpellStats(spellId, spellStats, summaryRow);
            tinsert(tableData, row);
        end
    end
    for unitName, healDoneByUnit in pairs(mechanics) do
        if (healDoneByUnit.meta and (healDoneByUnit.meta.unitName == name or healDoneByUnit.meta.unitName == nameAndRealm) and healDoneByUnit.meta.unitName ~= unitName) then
            for spellId, spellStats in pairs(healDoneByUnit.spells or healDoneByUnit) do
                if (spellId ~= "meta") then
                    local row = proceedSpellStats(spellId, spellStats, summaryRow, unitName);
                    tinsert(tableData, row);
                end
            end
        end
    end
    summaryRow.cols[8].value = summaryRow.cols[9].value / summaryRow.cols[4].value;
    for _, tableRow in pairs(tableData) do
        tableRow.cols[6].value = tableRow.cols[5].value / summaryRow.cols[5].value;
    end
    tinsert(tableData, summaryRow);
    return tableData, summaryRow;
end

--[[--
@param[type=table] row
@param[type=table] cols
@param[type=number] challengeId
@param[type=unitId] unitId
@return[type=table]
]]
function MyDungeonsBook:HealByPartyMemberBySpellFrame_Report_Create(row, cols, challengeId, unitId)
    local spellLink = GetSpellLink(row.cols[1].value);
    local challenge = self:Challenge_GetById(challengeId);
    local title = string.format(L["MyDungeonsBook Heal stats by %s with %s:"], challenge.players[unitId].name or "", spellLink);
    return self:Table_PlayersRowBySpell_Report_Create(row, cols, title);
end

function MyDungeonsBook:HealByEnemiesFrame_GetHeadersForTable(challengeId)
return {
        {
            name = " ",
            width = 1,
            align = "LEFT"
        },
        {
            name = "",
            width = 40,
            align = "LEFT",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsSpellIcon(...);
            end
        },
        {
            name = L["Spell"],
            width = 110,
            align = "LEFT",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsSpellLink(...);
            end
        },
        {
            name = L["Hits"],
            width = 40,
            align = "RIGHT"
        },
        {
            name = L["Amount"],
            width = 60,
            align = "RIGHT",
            sort = "asc",
            bgcolor = {
                r = 0,
                g = 0,
                b = 0,
                a = 0.4
            },
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsNumber(...);
            end
        },
        {
            name = L["%"],
            width = 40,
            align = "RIGHT",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsPercents(...);
            end
        },
        {
            name = L["Over"],
            width = 50,
            align = "RIGHT",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsNumber(...);
            end
        },
        {
            name = L["Crit, %"],
            width = 50,
            align = "RIGHT",
            bgcolor = {
                r = 0,
                g = 0,
                b = 0,
                a = 0.4
            },
            color = {
                r = 1,
                g = 0,
                b = 0,
                a = 1
            },
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsPercents(...);
            end
        },
        {
            name = L["Hits Crit"],
            width = 50,
            align = "RIGHT",
            bgcolor = {
                r = 0,
                g = 0,
                b = 0,
                a = 0.4
            },
        },
        {
            name = L["Max Crit"],
            width = 50,
            align = "RIGHT",
            bgcolor = {
                r = 0,
                g = 0,
                b = 0,
                a = 0.4
            },
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsNumber(...);
            end
        },
        {
            name = L["Min Crit"],
            width = 50,
            align = "RIGHT",
            bgcolor = {
                r = 0,
                g = 0,
                b = 0,
                a = 0.4
            },
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsNumber(...);
            end
        },
        {
            name = L["Max Not Crit"],
            width = 50,
            align = "RIGHT",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsNumber(...);
            end
        },
        {
            name = L["Min Not Crit"],
            width = 50,
            align = "RIGHT",
            DoCellUpdate = function(...)
                self:Table_Cell_FormatAsNumber(...);
            end
        }
    };
end

--[[--
@param[type=table] row
@param[type=table] cols
@param[type=number] challengeId
@param[type=unitId] unitId
@return[type=table]
]]
function MyDungeonsBook:HealByEnemiesFrame_Report_Create(row, cols, challengeId)
    local spellLink = GetSpellLink(row.cols[1].value);
    local challenge = self:Challenge_GetById(challengeId);
    local title = string.format(L["MyDungeonsBook Enemies Heal stats by %s:"], spellLink);
    return self:Table_PlayersRowBySpell_Report_Create(row, cols, title);
end
